/************************************************************

  grid.cpp

  The grid class for konnect4
  A Connect4 type of program.
  Gihan Marasingha
  30th December 1999

************************************************************/

#include <string.h>
#include <stdio.h>

/*
  The grid class is an abstraction of the playing grid.
  The width and height correspond to the physical widths and
  heights.

  `array' is an array of character pointers. Each element in
  the array contains a (physical) column of the grid. Each
  entry in a column contains the appropriate token
  ( O's or X's ).
  `size' is an array of integers, containing the number of
  filled spaces in each column of the array.
  clear is used to clear the grid.
  print prints it out to stdout.
  addtoken adds a new token to the given column. It returns
  `success' on success, `failure' on failure (failure
  occurs if there were no free spaces on the given column),
  and it returns `won' if the player wins by that move.
*/

#include "grid.h"

grid::grid(int w, int h)
{
  int i;

  width = (w > 0 ? w : 1);
  height = (h > 0 ? h : 1);
  array = new char *[width];
  size = new int[width];

  for(i=0;i<width;i++)
    array[i] = new char[height];

  clear();
}

grid::~grid()
{
  int i;

  for(i=0;i<width;i++)
    delete array[i];

  delete array;
  delete size;
}

void grid::clear(void)
{
  int i,j;
  for (i=0;i<width;i++)
    {
      size[i] = 0; // no entries in the ith column.
      for(j=0;j<height;j++)
	array[i][j] = ' ';
    }

}

void grid::print(void)
{
  int i,j;

  printf("\n");
  for(i=0;i<width;i++)
    printf(" %d%s",i,(i<9 ? " " : ""));
  for(j=height-1;j>=0;j--)
    {
      printf("\n");
      for(i=0;i<width;i++)
	printf("[%c]",array[i][j]);
    }
  printf("\n");
}

int grid::addtoken(int col, char token)
{
  if( col < 0 || col >= width)
    return failure;
  if (size[col]==height)
    return failure;

  array[col][size[col]] = token;
  (size[col])++;

  return haswon(col,size[col]-1, token);
}

int grid::haswon(int col, int row, char token)
{
  int dx[4] = {1, 0, +1, 1};
  int dy[4] = {0, 1, -1, 1};
  int i;

  for(i=0;i<4;i++)
	 if(haswondir(col,row,token,dx[i],dy[i])==win)
		return win;

  return success;
}

int grid::haswondir(int col, int row, char token, int dx, int dy)
{
  int x,y,i;

  x = col; y = row;

  while (x>=0 && x<width && y>=0 && y<height)
  {
	  if(array[x][y] != token)
		 break;
	  x -= dx;
	  y -= dy;
  }

  x += dx;
  y += dy;

  for(i=0;i<4;i++)
	 {
		if (x>=0 && x<width && y>=0 && y<height)
	{
	  if(array[x][y] != token)
		 return success;
	}
		else
	return success;

		x += dx;
		y += dy;
	 }

  return win;
}







